//+------------------------------------------------------------------+
//| P-Williams.mq4
//| Copyright  2009, ForexArea.com
//| Author: Steve Fleming
//| http://www.forexarea.com; http://www.automatedtradingsoftware.com
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, MetaQuotes Software Corp."
#property link      "http://www.forexarea.com; http://www.automatedtradingsoftware.com"

// This EA uses the OrderReliable set of functions.
// If you do not wish to use this simply remove the word 'Reliable' from the functions below
// and re-compile.

// OrderReliable.mqh MUST be placed in experts/include folder
// IGNORE any warnings as they are normal and have present no problem
#include <OrderReliable.mqh>

extern int     TAKEPROFIT     = 100;
extern int     STOPLOSS       = 50;
extern int     TRAILINGSTOP   = 25; // set to zero (0) if you don't want to trail
extern int     TS_TRIGGER     = 35; // pips in profit before trailing stop is triggered
extern int     SLIPPAGE       = 3; // slippage 
extern double  RISK           = 1; // percentage of account to risk
extern int     MAGIC          = 21114444; // uid to identiy this EA (can be anything as long as it's not the same as another EA)
extern int     SHIFT          = 0; // 0:current bar, 1:closed bar 

double ExtUpperBuffer, ExtLowerBuffer;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
//---- 2 allocated indicator buffers
    SetIndexBuffer(0,ExtUpperBuffer);
    SetIndexBuffer(1,ExtLowerBuffer);
//---- drawing parameters setting
    SetIndexStyle(0,DRAW_ARROW);
    SetIndexArrow(0,217);
    SetIndexStyle(1,DRAW_ARROW);
    SetIndexArrow(1,218);
//---- displaying in the DataWindow
    SetIndexLabel(0,"Fractal Up");
    SetIndexLabel(1,"Fractal Down");
//---- initialization done  
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----
   int iHeikenSignal;
   
   // calculate lots based on account margin and risk
   double dRisk = AccountFreeMargin()*RISK/100.0;
   double dRisk1 = MarketInfo(Symbol(), MODE_TICKVALUE)*STOPLOSS;
   double dLots = FixVolume(dRisk/dRisk1);
   //
   
   
   if(NewBar() == true){
      int iOrderCnt = CountOrders();
        /*
      if(iOrderCnt == 1){
         // see if we have an opposite signal and if so close the open order and open an new one
         iHeikenSignal = HHSignal();
         int iOrderType = GetOrderType();
         switch (iOrderType){
            case OP_BUY:
               if(iHeikenSignal == OP_SELL){
                  // close buy order 
                  CloseOrder();
                  // open sell order 
                  OpenSell(dLots);
               }   
               break;
            case OP_SELL:
               if(iHeikenSignal == OP_BUY){
                  // close sell order 
                  CloseOrder();
                  // open buy order 
                  OpenBuy(dLots);
               }
               break;
            default:
               break;      
         }
      }else{*/
      if(iOrderCnt == 0){
         //open our initial order on change of direction
         iHeikenSignal = HHSignal();
         Print(HHSignal());
         if(iHeikenSignal == OP_BUY) OpenBuy(dLots);
         if(iHeikenSignal == OP_SELL ) OpenSell(dLots);
      }
   } //end of NewBar if statement    
   
   // do trailing stop if necessary
   if(TRAILINGSTOP != 0)ManageTrailingStop();
   
//----
   return(0);
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| close order function                                             |
//+------------------------------------------------------------------+
void CloseOrder(){
   for(int i=0;i<OrdersTotal();i++){
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGIC)
      {
         // found an open BUY/SELL order 
         if(OrderType() == OP_BUY || OrderType() == OP_SELL){
            // we should only have one open order so just close it
            switch(OrderType()){
               case OP_BUY:
                  OrderCloseReliable(OrderTicket(),OrderLots(),Bid,SLIPPAGE,Red);
                  break;
               case OP_SELL:
                  OrderCloseReliable(OrderTicket(),OrderLots(),Ask,SLIPPAGE,Red);
                  break;
               default:
                  break;      
            }
         }
      }
   }
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| open sell function                                               |
//+------------------------------------------------------------------+
void OpenSell(double ldLots){
   double dStopLoss, dTakeProfit;
   color cColor;

   if(STOPLOSS > 0)dStopLoss = Bid + STOPLOSS*Point;
   if(TAKEPROFIT > 0)dTakeProfit = Bid - TAKEPROFIT*Point;
   cColor=DeepPink;

   OrderSendReliable(Symbol(),OP_SELL,ldLots,Bid,SLIPPAGE,dStopLoss,dTakeProfit, MAGIC + " SELL",MAGIC,cColor); 
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| open buy function                                                |
//+------------------------------------------------------------------+
void OpenBuy(double ldLots){
   double dStopLoss, dTakeProfit;
   color cColor;

   if(STOPLOSS > 0)dStopLoss = Ask - STOPLOSS*Point;
   if(TAKEPROFIT > 0)dTakeProfit = Ask + TAKEPROFIT*Point;
   cColor=DodgerBlue;

   OrderSendReliable(Symbol(),OP_BUY,ldLots,Ask,SLIPPAGE,dStopLoss,dTakeProfit,MAGIC + " BUY",MAGIC,cColor); 
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| heiken ashi function                                             |
//+------------------------------------------------------------------+
int HHSignal(){
   double dPreviousSignal1, dPreviousSignal2, dClosedSignal1, dClosedSignal2;
   dPreviousSignal1 = iCustom(Symbol(), 0, "Heiken Ashi",3,SHIFT+1);
   dPreviousSignal2 = iCustom(Symbol(), 0, "Heiken Ashi",2,SHIFT+1);
   dClosedSignal1 = iCustom(Symbol(), 0, "Heiken Ashi",3,SHIFT);
   dClosedSignal2 = iCustom(Symbol(), 0, "Heiken Ashi",2,SHIFT);
   
   // direction has changed to down
   if(dPreviousSignal1 > dPreviousSignal2 && dClosedSignal1 < dClosedSignal2){
      return(OP_SELL);}
   // direction has changed to up
   if(dPreviousSignal1 < dPreviousSignal2 && dClosedSignal1 > dClosedSignal2){
      return(OP_BUY);}

   // direction is unchanged
   return(-1);
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| fix volume function                                              |
//+------------------------------------------------------------------+
double FixVolume(double dLots){
   // Adjust trade volume to broker
   double dStep = MarketInfo(Symbol(), MODE_LOTSTEP);
   double dMin = MarketInfo(Symbol(), MODE_MINLOT);
   double dMax = MarketInfo(Symbol(), MODE_MAXLOT);
   double dResult = 0;

   if(dMax > 0)
    dLots = MathMin(dLots, dMax);
   if(dStep > 0)
    dResult = MathFloor(dLots/dStep)*dStep;
   else
    dResult = 0;
 
   if(dResult < dMin)
    if(dMin>0)
      return(dMin);
    else
      return(dLots);
   else if(dResult > 0)
    return(dResult);
   else
    return(dLots);
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| order type function                                              |
//+------------------------------------------------------------------+
int GetOrderType(){
   // return the order type of our ONLY open order 
   for(int i=0;i<OrdersTotal();i++){
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGIC)
      {
         // found an open BUY/SELL order 
         if(OrderType() == OP_BUY || OrderType() == OP_SELL){
            return(OrderType());
         }
      }
   }
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| count orders function                                            |
//+------------------------------------------------------------------+
int CountOrders(){
   // return count of open orders 
   int iCnt = 0;
   for(int i=0;i<OrdersTotal();i++){
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGIC)
      {
         // found an open BUY/SELL order 
         if(OrderType() == OP_BUY || OrderType() == OP_SELL){
            iCnt++;
         }
      }
   }
   return(iCnt);     
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| new bar function                                                 |
//+------------------------------------------------------------------+
bool NewBar() {
   // check for new bar 
	static datetime LastTime = 0;

	if (Time[0] != LastTime) {
		LastTime = Time[0];		
		return (true);
	} else
		return (false);
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| trailing stop function                                           |
//+------------------------------------------------------------------+
void ManageTrailingStop(){
   
   for(int i=0;i<OrdersTotal();i++){
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGIC )
      {
         // found an open BUY/SELL order 
         switch(OrderType()){
            case OP_BUY:
               if((Bid - OrderOpenPrice() >= TS_TRIGGER*Point) &&  OrderStopLoss() < Bid-TRAILINGSTOP*Point){
                  OrderModifyReliable(OrderTicket(),OrderOpenPrice(),Bid-TRAILINGSTOP*Point,OrderTakeProfit(),0,Blue);
               }
               break;
            case OP_SELL:
               if(OrderOpenPrice()-Ask >= TS_TRIGGER*Point && OrderStopLoss() > Ask + TRAILINGSTOP*Point){
                  OrderModifyReliable(OrderTicket(),OrderOpenPrice(),Ask + TRAILINGSTOP*Point,OrderTakeProfit(),0,Red);
               } 
               break;
            default:
               Print("An unrecognized order type has been passed to ManageTrades()");
               break;      
         }
      }
   }    
}
//+------------------------------------------------------------------+